--==============================================================================
-- CERN (BE-CO-HT)
-- Xilinx MultiBoot FSM
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2013-08-19
--
-- version: 1.0
--
-- description: 
--
-- dependencies:
--
-- references:
-- 
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2013-08-19   Theodor Stana     t.stana@cern.ch     File created
--==============================================================================
-- TODO: - 
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity multiboot_fsm is
  port
  (
    -- Clock and reset inputs
    clk_i           : in  std_logic;
    rst_n_i         : in  std_logic;

    -- Control register inputs
    rdbootsts_i     : in  std_logic;
    wmb_i           : in  std_logic;
    wgb_i           : in  std_logic;
    iprog_i         : in  std_logic;

    -- Multiboot and golden bitstream start addresses
    gbbar_i         : in  std_logic_vector(31 downto 0);
    mbbar_i         : in  std_logic_vector(31 downto 0);

    -- Outputs to status register
    bootsts_img_o   : out std_logic_vector(15 downto 0);
    bootsts_valid_o : out std_logic;

    -- Ports for the external ICAP component
    icap_dat_i      : in  std_logic_vector(15 downto 0);
    icap_dat_o      : out std_logic_vector(15 downto 0);
    icap_ce_n_o     : out std_logic;
    icap_wr_n_o     : out std_logic
  );
end entity multiboot_fsm;


architecture behav of multiboot_fsm is

  --============================================================================
  -- Type declarations
  --============================================================================
  type t_state is
    (
      IDLE,
      DUMMY,
      SYNC_H,
      SYNC_L,
      GEN_1,
      MBA_L,
      GEN_2,
      MBA_H,
      GEN_3,
      GBA_L,
      GEN_4,
      GBA_H,
      CMD,
      IPROG,
      NOP
    );

  --============================================================================
  -- Constant declarations
  --============================================================================

  --============================================================================
  -- Component declarations
  --============================================================================

  --============================================================================
  -- Signal declarations
  --============================================================================
  signal state     : t_state;

  signal icap_din  : std_logic_vector(15 downto 0);
  signal icap_dout : std_logic_vector(15 downto 0);
  signal icap_ce_n : std_logic;
  signal icap_wr_n : std_logic;

--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- I/O logic
  --============================================================================
  icap_din    <= icap_dat_i;
  icap_dat_o  <= icap_dout;
  icap_ce_n_o <= icap_ce_n;
  icap_wr_n_o <= icap_wr_n;

  --============================================================================
  -- FSM logic
  --============================================================================
  p_fsm : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if (rst_n_i = '0') then
        state     <= IDLE;
        icap_dout <= (others => '0');
        icap_ce_n <= '1';
        icap_wr_n <= '1';
      else
        case state is

          when IDLE =>
            icap_ce_n <= '1';
            icap_wr_n <= '1';
            if (iprog_i = '1') then
              state     <= DUMMY;
              icap_ce_n <= '0';
              icap_wr_n <= '0';
            end if;

          --====================================================================
          -- IPROG sequence
          --====================================================================
          when DUMMY =>
            icap_dout <= x"ffff";
            state     <= SYNC_H;

          when SYNC_H =>
            icap_dout <= x"aa99";
            state     <= SYNC_L;

          when SYNC_L =>
            icap_dout <= x"5566";
            state     <= GEN_1;

          when GEN_1 =>
            icap_dout <= x"3261";
            state     <= MBA_L;

          when MBA_L =>
            icap_dout <= mbbar_i(15 downto 0);
            state     <= GEN_2;

          when GEN_2 =>
            icap_dout <= x"3281";
            state     <= MBA_H;

          when MBA_H =>
            icap_dout <= mbbar_i(31 downto 16);
            state     <= GEN_3;

          when GEN_3 =>
            icap_dout <= x"32a1";
            state     <= GBA_L;

          when GBA_L =>
            icap_dout <= gbbar_i(15 downto 0);
            state     <= GEN_4;

          when GEN_4 =>
            icap_dout <= x"32c1";
            state     <= GBA_H;

          when GBA_H =>
            icap_dout <= gbbar_i(31 downto 16);
            state     <= CMD;

          when CMD =>
            icap_dout <= x"30a1";
            state     <= IPROG;

          when IPROG =>
            icap_dout <= x"000e";
            state     <= NOP;

          when NOP =>
            icap_dout <= x"2000";
            state     <= IDLE;

        end case;
      end if;
    end if;
  end process p_fsm;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
